


#include <iostream>



#include "soma/utils/general/IsOfType.hpp"
#include "soma/utils/general/BitArray3D.hpp"
#include "soma/solver/soma_structure/SomaLoader.hpp"
#include "soma/solver/soma_structure/AllPolyformPiecesPositions.hpp"
#include "soma/solver/soma_structure/AllSomaPiecesPositions.hpp"
#include "soma/solver/dlx_algorithm/DLXAlgorithm.hpp"
#include "soma/solver/dlx_algorithm/DLXSomaSolver.hpp"



int main()
  {

  bool somaRunning=true,
       displayBeginMessage=true;
  std::string fileName,
              yesOrNo;

  do
    {

    displayBeginMessage=true;

    try
      {

      std::cout<<std::endl<<"  Veuillez entrer le nom du fichier contenant les coordonnees de la structure a resoudre :  ";
      std::cin>>fileName;

//  On charge la structure  rsoudre en passant le nom du fichier o ses coordonnes se trouvent au constructeur de
//  l'objet somaLoader de la classe CSomaLoader.
      soma::solver::soma_structure::CSomaLoader somaLoader
                                                         (
                                                         fileName.c_str()
                                                         );

//  On calcule toutes les positions possibles dans la stucture pour chacunes des pices A, B, L, P, T, V et Z.
      soma::solver::soma_structure::CAllSomaPiecesPositions somaPositions
                                                                        (
                                                                        somaLoader.getBoundingBox()
                                                                        );

//  Si la structure est de parit autre que -5, -3, -1, 1, 3 et 5 alors on est sur que la structure n'a pas de solution. hasNoSolution retourne alors true.
//  De mme, si les 27 cubes lus auparavant ne sont pas tous de coordonnes diffrentes, alors hasNoSolution retoune true galement car la structure n'est
//  en fait pas constitue de 27 cubes.
//  Autrement hasNoSolution renvoie false et on ne peut rien conclure; il faut alors lancer l'algorithme DLX.
      if(!somaPositions.hasNoSolution())
        {

        unsigned int maxNbSolutions=std::numeric_limits<unsigned int>::max();

        std::cout<<std::endl<<"  Voulez-vous que l'algorithme DLX tente de trouver toutes les solutions de la structure, s'il y en a?"<<std::endl<<"  Entrez y ou o pour oui, ou n pour non :  ";
        std::cin>>yesOrNo;
        while((yesOrNo!="o")&&(yesOrNo!="O")&&(yesOrNo!="y")&&(yesOrNo!="Y")&&(yesOrNo!="n")&&(yesOrNo!="N"))
         {

         std::cout<<std::endl<<"  Entrez y ou o pour oui, ou n pour non :  ";
         std::cin>>yesOrNo;

         }
        if((yesOrNo=="n")||(yesOrNo=="N"))
          {

          std::string maxNbSolutionsString;
          std::cout<<std::endl<<" Combien de solutions souhaitez-vous que l'algorithme DLX calcule au maximum?  ";
          std::cin>>maxNbSolutionsString;

          int tmpVal;
          soma::utils::general::isOfType<int>
                                            (
                                            tmpVal=0,
                                            maxNbSolutionsString
                                            );

          while(tmpVal<1)
            {

            std::cout<<std::endl<<"  Veuillez entrer un entier > 0 :  ";
            std::cin>>maxNbSolutionsString;
            soma::utils::general::isOfType<int>
                                              (
                                              tmpVal=0,
                                              maxNbSolutionsString
                                              );

            }

          maxNbSolutions=tmpVal;

          }

//  On cre une instance de l'algorithme DLX avec le nombre maximum de solution que l'on souhaite calculer.
        soma::solver::dlx_algorithm::CDLXSomaSolver somaSolver
                                                             (
                                                             somaPositions.getAllPiecesPositions(),maxNbSolutions
                                                             );

//  La fonction search lance l'algorithme DLX en question.
        somaSolver.search();

//  Si l'algorithme DLX a converg, alors hasSolution retourne true.
        if(somaSolver.hasSolution())
          {

//  On affiche le nombre de solutions trouves :

          const unsigned int nbSolutions=somaSolver.getNumberOfFoundSolutions();

          if(nbSolutions==1)
            std::cout<<std::endl<<"  L'algorithme DLX a trouve la solution!"<<std::endl;
          else
            std::cout<<std::endl<<"  L'algorithme DLX a trouve "<<somaSolver.getNumberOfFoundSolutions()<<" solutions!"<<std::endl;

          bool wantToDisplayASolution=true;

          do
            {

            if(nbSolutions==1)
              std::cout<<std::endl<<"  Voulez-vous afficher la solution?"<<std::endl<<"  Entrez y ou o pour oui, ou n pour non :  ";
            else
              std::cout<<std::endl<<"  Voulez-vous afficher une solution?"<<std::endl<<"  Entrez y ou o pour oui, ou n pour non :  ";
            std::cin>>yesOrNo;
            while((yesOrNo!="o")&&(yesOrNo!="O")&&(yesOrNo!="y")&&(yesOrNo!="Y")&&(yesOrNo!="n")&&(yesOrNo!="N"))
              {

              std::cout<<std::endl<<"  Entrez y ou o pour oui, ou n pour non :  ";
              std::cin>>yesOrNo;

              }

            if((yesOrNo=="o")||(yesOrNo=="O")||(yesOrNo=="y")||(yesOrNo=="Y"))
              {

              int solutionIndex=1;

              if(nbSolutions>1)
                {
                std::string solutionIndexString;

                std::cout<<std::endl<<" Qelle solution voulez-vous afficher?  ";
                std::cin>>solutionIndexString;

                soma::utils::general::isOfType<int>
                                                  (
                                                  solutionIndex=0,
                                                  solutionIndexString
                                                  );

                while((solutionIndex<1)||(solutionIndex>(int)nbSolutions))
                  {

                  std::cout<<std::endl<<"  Veuillez entrer un entier compris entre 1 et "<<nbSolutions<<" (bornes incluses) :  ";
                  std::cin>>solutionIndexString;
                  soma::utils::general::isOfType<int>
                                                    (
                                                    solutionIndex=0,
                                                    solutionIndexString
                                                    );

                  }
                }

//  Pour afficher une solution, on rcupre la std::map retourne par getIthSolution.
//  Les clefs sont de type char et correspondent aux pices du jeu SOMA. Les valeurs sont
//  des std::vector contenant les positions des 3 (pour V) ou 4 (pour les autres) des cubes de la structure.
//  Comme ceux-ci sont repres relativement  la bote englobante de la structure, il suffit de crer un
//  tableau de char 3 dimensions (les mmes que celles de la bote englobante de la structure) dans lequel un
//  lment reoit le nom d'une pice (A, B, L, P, T, V ou Z) chaque fois que son indice correspond  une
//  position d'un cube de la pice. On affiche ensuite chaque couche de la bote englobante sous forme d'un
//  tableau de char  2 dimensions dans le plan (0YZ), ceci suivant les valeurs croissantes de X.

              std::map<char,std::vector<unsigned int> > ithSolution=somaSolver.getIthSolution
                                                                                            (
                                                                                            solutionIndex-1
                                                                                            );

//  On dclare le tableau de char correspondant  la bote englobante de la structure. Comme le cube de SOMA est
//  la seule structure  occuper toute sa bote englobante, on affiche le charactre '-' aux positions de la
//  bote englobante o on n'affiche pas le nom d'une pice.
              std::vector<char> structureBoundingBox
                                                   (
                                                   somaLoader.getBoundingBox().size(),
                                                   '-'
                                                   );

              for(std::map<char,std::vector<unsigned int> >::const_iterator ithSolIter=ithSolution.begin();ithSolIter!=ithSolution.end();++ithSolIter)
                {

                const char pieceName=ithSolIter->first;

//  On place le nom de la pice dans le tableau char  chaque position donn"e par sa liste.
                const unsigned int nbCubesInPiece=ithSolIter->second.size();
                for(unsigned int i=0;i<nbCubesInPiece;++i)
                  structureBoundingBox[ithSolIter->second[i]]=pieceName;

                }

              const unsigned int N1=somaLoader.getBoundingBox().size1(),
                                 N2=somaLoader.getBoundingBox().size2(),
                                 N3=somaLoader.getBoundingBox().size3();

              for(unsigned int i=0;i<N1;++i)
                {

//  On affiche chaque couche de la bote englobante sous forme de tableau bi-dimensionnel dans le plan (0YZ), suivant
//  les valeurs successives de X.
                std::cout<<std::endl<<"  Couche "<<(i+1)<<" :"<<std::endl;

                for(unsigned int j=0;j<N2;++j)
                  {

                  std::cout<<std::endl;

                  for(unsigned int k=0;k<N3;++k)
                    std::cout<<"  "<<structureBoundingBox[N3*(N2*i+j)+k];//  On utilise une formule classique pour convertir les indices
                                                                         //  uni-dimensionnels en indices tri-dimensionnels (voir http://en.wikipedia.org/wiki/Row-major_order).
                  }

                std::cout<<std::endl;

                }

              if(nbSolutions==1)
                wantToDisplayASolution=false;

              }

            else
              wantToDisplayASolution=false;

            } while(wantToDisplayASolution);

          }

        else
          std::cout<<std::endl<<"  La structure n'a pas de solution!"<<std::endl;
        }

      else
        {
        std::cout<<std::endl<<"  La structure n'a pas de solution car sa parite est differente de -5, -3, -1, 1, 3 et 5!"<<std::endl;
        }

      }

    catch(const std::out_of_range& e)
      {
      std::cerr<<std::endl<<"  "<<e.what()<<std::endl;
      }

    catch(const std::invalid_argument& e)
      {
      std::cerr<<std::endl<<"  "<<e.what()<<std::endl;
      }

    catch(const std::runtime_error& e)
      {
      std::cerr<<std::endl<<"  Le fichier specifie n'existe pas ou n'est pas accessible en lecture!"<<std::endl;
      displayBeginMessage=false;
      }

    catch(const std::exception& e)
      {
      std::cerr<<std::endl<<"  "<<e.what()<<std::endl;
      return
           (
           -1
           );
      }

    if(displayBeginMessage)
      {
      std::cout<<std::endl<<"  Voulez-vous tenter de resoudre une autre structure?"<<std::endl<<"  Entrez y ou o pour oui, ou n pour non :  ";
      std::cin>>yesOrNo;
      while((yesOrNo!="o")&&(yesOrNo!="O")&&(yesOrNo!="y")&&(yesOrNo!="Y")&&(yesOrNo!="n")&&(yesOrNo!="N"))
        {

        std::cout<<std::endl<<"  Entrez y ou o pour oui, ou n pour non :  ";
        std::cin>>yesOrNo;

        }
      if((yesOrNo=="n")||(yesOrNo=="N"))
        {

        std::cout<<std::endl<<"  Merci d'avoir utilise ce programme."<<std::endl<<"  A bientot!";
        somaRunning=false;

        }
      }

    } while(somaRunning);

  return
       (
       0
       );
  }
