#ifndef ALL_POLYFORM_PIECES_POSITIONS_HPP
#define ALL_POLYFORM_PIECES_POSITIONS_HPP



#include <vector>
#include "../../utils/general/BitArray3D.hpp"



namespace soma
  {



  namespace utils
    {

    namespace general
      {

//  Dclaration anticipe de la classe CBitArray3D. C'est inutile ici car un membre de la classe
//  CSomaLoader est de type CBitArray3D (composition). C'est plus pour indiquer qu'on va utiliser
//  la classe CBitArray3D.
      class CBitArray3D;

      }

    }



  namespace solver
    {



    namespace soma_structure
      {



/*
  CAllPolyformPiecesPositions est une classe abstraite (au sens du CRTP). En effet, la fonction virtuelle
  (au sens du CRTP) doit tre redfinie par drivation dans une classe fille de CAllPolyformPiecesPositions,
  afin de calculer toutes les positions possibles de toutes les pices d'un certain jeu dans une structure
  quelconque.
*/

      template
        <
        unsigned char NB_PIECES,
        typename DerivedT,
        typename BitArrayT=soma::utils::general::CBitArray3D,
        typename PositionsMatrixT=std::vector<std::vector<bool> >
        >
      class CAllPolyformPiecesPositions
        {

        private :

//  Copie d'un objet de type CAllPolyformPiecesPositions interdite.
          CAllPolyformPiecesPositions
                                    (
                                    const CAllPolyformPiecesPositions&
                                    );



//  Assignation d'un objet de type CAllPolyformPiecesPositions interdite.
          void operator=
                       (
                       const CAllPolyformPiecesPositions&
                       );


        public :

          typedef BitArrayT bit_array_t;
          typedef PositionsMatrixT positions_matrix_t;


        protected :


/*
    Donne membre prive :
      - type : Drfrence constante sur un lment de type bit_array_t (pas de recopie inutile).
      - nom : m_BoundingBox.
      - m_BoundingBox est la bote englobante contenant la structure une fois charge. Elle est centre en (0,0,0). Elle est code par un tableau de boolens
         3 dimensions. Si un cube appartenant  la structure est prsent  la position (i,j,k), alors m_BoundingBox(i,j,k)==true. Sinon, m_BoundingBox(i,j,k)==false.
        En gnral, m_BoundingBox est initialis (via le constructeur de sa classe), par un un lment de type soma::utils::general::CBitArray3D founi par la classe
        CSomaLoader via sa fonction membre getBoundingBox pour le jeu SOMA.
*/
          const bit_array_t& m_BoundingBox;

/*
    Donne membre prive :
      - type : bool.
      - nom : m_HasNoSolution.
      - Pour le jeu SOMA, m_HasNoSolution vaut true si la structure comporte des cubes de coordonnes diffrentes ou est de parit autre que -5, -3, -1, 1, 3 et 5 car on
        est sur alors que la srtucture n'a pas de solution. Attention, quand m_HasNoSolution vaut false, a ne veut pas dire que la structure a une solution;
        il faut lancer un algorithme de rsolution pour le savoir.
*/
          bool m_HasNoSolution;

/*
    Donne membre prive :
      - type : positions_matrix_t (en gnral, c'est un alias de std::vector<std::vector<bool> >).
      - nom : m_PiecesPositionsLists.
      - m_PiecesPositionsLists est une matrice boolenne contenant toutes les positions possibles des pices d'un jeu dans une structure.
*/
          positions_matrix_t m_PiecesPositionsLists;



/*
   La fonction membre protge computeAllPiecesPositions est une fonction virtuelle au sens du CRTP. C'est  elle qu'incombe la tche de
   calculer les positions possibles des pices d'un jeu dans une structure.

   Argument : aucun.
   Retour : void.
*/
          inline void computeAllPiecesPositions()
            {
            (static_cast<DerivedT*>(this))->computeAllPiecesPositions();
            }


        public :

/*
    Donne membre publique, statique et constante.
      - type : const unsigned char.
       - nom : NUMBER_OF_PIECES.
       C'est le nombre de pices du jeu, soit 7 pour le jeu SOMA.
*/
          static const unsigned char NUMBER_OF_PIECES=NB_PIECES;



/*
     Unique constructeur public de la classe CAllPolyformPiecesPositions. Reoit la bote englobante de la structure  rsoudre.

     Argument :
       - type : const bit_array_t& boundingBox. En gnral, bit_array_t est un alias de soma::utils::general::CBitArray3D.
       - nom : boundingBox.
       - boundingBox est la bote englobante de la structure  rsoudre.
*/
          CAllPolyformPiecesPositions
                                    (
                                    const bit_array_t& boundingBox
                                    );



/*
    getAllPiecesPositions est une fonction membre publique const.

    Argument : Aucun.

    Retour :
      - type : positions_matrix_t (en gnral, c'est un alias de std::vector<std::vector<bool> >).
      - nom : m_PiecesPositionsLists.
      - Cette fonction membre publique est l'accesseur au membre m_PiecesPositionsLists.
*/
          inline const positions_matrix_t& getAllPiecesPositions() const
            {
            return
                 (
                 m_PiecesPositionsLists
                 );
            }



/*
    hasNoSolution est une fonction membre publique const.

    Argument : Aucun.

    Retour :
      - type : const bool.
      - nom : m_HasNoSolution.
      - Cette fonction membre publique est l'accesseur au membre m_HasNoSolution.
*/
          inline const bool hasNoSolution() const
            {
            return
                 (
                 m_HasNoSolution
                 );
            }

        };



      #include "AllPolyformPiecesPositions.tpp"



      }



    }



  }



#endif  //  ALL_POLYFORM_PIECES_POSITIONS_HPP
