



namespace soma
  {



  namespace solver
    {



    namespace soma_structure
      {



/*
  La fonction prive addNewPosToVList ajoute une nouvelle ligne newRow  la matrice boolenne de l'ensemble
  des positions des pices du jeu SOMA dans une certaine structure S. Cette nouvelle ligne compte
  NUMBER_OF_PIECES+N1xN2xN3 colonnes o NUMBER_OF_PIECES est le nombre de pices du jeu que l'on traite
  (ici c'est toujours 7 mais NUMBER_OF_PIECES est hrit de la classe mre CAllPolyformPiecesPositions, plus gnrale).
  Quant  N1, N2 et N3, ce sont le nombre de cubes sur l'axe des X, des Y et des Z, respectivement, dans la bote
  englobante de S. De plus, newRow correspond  une nouvelle position de la pice V dans S calcule par la fonction
  computeAllPiecesPositions. Sur la sixime colonne (numrote 5) de newRow, addNewPosToVList place un 1 car V est
  la sixime pice du jeu SOMA dans l'ordre alphabtique (on convient que c'est arbtraire...). 3 autres colonnes de
  newRow, situes aprs la septime colonne, sont assignes  1 (toutes les autres sont assignes  0). Elles indiquent
  quels cubes de la bote englobante la nouvelle position calcule est permise  V d'occuper dans S. Notons que les
  positions des 3 cubes sont transmises sous forme de 3 3-upls (i1,j1,k1), (i2,j2,k2) et (i3,j3,k3) alors que les
  colonnes de newRow, aprs la septime, sont numrotes de 0  N1xN2xN3-1. En fait, on utilise une formule classique
  pour passer d'un indiage de tableau tri-dimensionnel  uni-dimensionnel (voir http://en.wikipedia.org/wiki/Row-major_order).
  La pice V est traite  part des autres pices du jeu SOMA car c'est la seule  tre constitue de 3 pices quans les
  six autres sont formes de 4 pices.
  Enfin, pourquoi addNewPosToVList est statique? Et pourquoi pas? Plus srieusement, il parat qu'il est bon d'employer
  des fonctions statiques quand c'est possible car cela permet  C++de n'enregistrer qu'une seule fonction par classe
  au lieu d'une pour chaque instance de la classe dans la table des fonctions. En mme temps, il n'est pas cens y avoir
  des milliers d'objets de type AllSomaPiecesPositions. Donc disons simplement que addNewPosToVList a t dclare
  statique histoire de varier les plaisirs...

  Arguments 1 et 2 :
    - type : integer_t (ici alias de unsigned short).
    - noms : N2 et N3.
    - N2 et N3 sont le nombre de cubes sur l'axe des Y et des Z, respectivement d'une bote englobante d'une structure S.
      Ils servent pour la formule de conversion d'indices de tableau  3 dimensions vers tableau  1 dimension.

  Argument 3 :
    - type : integer_t (ici alias de unsigned short).
    - nom : N1xN2xN3.
    - N1xN2xN3 est le nombre total de cubes dans la bote englobante  de S.

  Argument 4 :integer_t (ici alias de unsigned short).
    - type : rfrence sur MatrixT (ici alias de std::vector<std::vector<bool> >).
    - nom : somaMatrix.
    - somaMatrix est la matrice boolenne dont les lignes forment l'ensemble de toutes les positions possibles
      de toutes les pices du jeu SOMA dans S.

  Arguments 5  13 :
    - type : integer_t (ici alias de unsigned short).
    - noms : i1, j1, k1, i2, j2, k2, i3, j3 et k3.
    - Les 3 3-upls (i1,j1,k1), (i2, j2 ,k2) et (i3,j3,k3) correspondent aux positions de 3 cubes (diffrents
      sinon computeAllPiecesPositions n'appellerait pas addNewPosToVList...) dans la bote englobante de S.

  Retour : void.
*/
      template
        <
        typename IntegerT1,
        typename IntegerT2,
        typename MatrixT
        >
      inline void CAllSomaPiecesPositions::addNewPosToVList
                                                          (
                                                          const IntegerT1 N2,
                                                          const IntegerT1 N3,
                                                          const IntegerT2 N1xN2xN3,
                                                          MatrixT& somaMatrix,
                                                          const IntegerT1 i1,
                                                          const IntegerT1 j1,
                                                          const IntegerT1 k1,
                                                          const IntegerT1 i2,
                                                          const IntegerT1 j2,
                                                          const IntegerT1 k2,
                                                          const IntegerT1 i3,
                                                          const IntegerT1 j3,
                                                          const IntegerT1 k3
                                                          )
        {

// On cre une nouvelle ligne newRow comportant NUMBER_OF_PIECES+N1xN2xN3 colonnes.
        typename MatrixT::value_type newRow
                                          (
                                          NUMBER_OF_PIECES+N1xN2xN3,
                                          false
                                          );
// La sixime colonne de newRow ainsi que les 3 colonnes indiques par les 3 3-upls (i1,j1,k1), (i2, j2 ,k2) et (i3,j3,k3)
// sont assignes  1, toutes les autres tant assignes  0.
        newRow[5]=
        newRow[positions_matrix_t::size_type(NUMBER_OF_PIECES+N3*(N2*i1+j1)+k1)]=
        newRow[positions_matrix_t::size_type(NUMBER_OF_PIECES+N3*(N2*i2+j2)+k2)]=
        newRow[positions_matrix_t::size_type(NUMBER_OF_PIECES+N3*(N2*i3+j3)+k3)]=true;
// La ligne newRow est ajoute  la suite des autres lignes de la matrice boolenne somaMatrix.
        somaMatrix.push_back
                           (
                           newRow
                           );

        }



/*
  La fonction prive addNewPosToPieceList est trs similaire  addNewPosToVList car son rle est galement d'ajouter
  une nouvelle ligne newRow,  la matrice boolenne de l'ensemble des positions des pices du jeu SOMA dans une
  certaine structure S. newRow est aussi constitue de NUMBER_OF_PIECES+N1xN2xN3 colonnes. NUMBER_OF_PIECES est
  encore une fois le nombre de pices d'un jeu (ici 7) et N1, N2 et N3 sont toujours le nombre de cubes sur l'axe
  des X, des Y et des Z, respectivement, dans la bote englobante de S. A la diffrence de addNewPosToVList,
  la nouvelle position ajoute par addNewPosToPieceList concerne l'une des pices A, B, L, P, T ou Z du jeu SOMA.
  Ainsi, addNewPosToPieceList reoit comme argument supplmentaire l'indice de la pice en question : 0 pour A,
  1 pour B, 2 pour L, 3 pour P, 4 pour T et 6 pour Z (ordre alphabtique). Cet indice correspond  la colonne
  de newRow (de 0  6) qui sera assigne  1. chacune des pices A, B, L, P, T et Z occupant 4 cubes dans la bote
  englobante de S, 4 autres colonnes de newRow, situes aprs la septime colonne, sont aussi assignes  1
  (toutes les autres sont assignes  0). Ces positions des 4 cubes sont transmises sous forme de 4 3-upls
  (i1,j1,k1), (i2,j2,k2), (i3,j3,k3) et (i4,j4,k4) et sont converties en numro de colonne allant de 0  N1xN2xN3-1
  grce  une formule classique pour passer d'un indiage de tableau tri-dimensionnel  uni-dimensionnel
  (voir http://en.wikipedia.org/wiki/Row-major_order).

  Argument 1 et 2 :
    - type : integer_t (ici alias de unsigned short).
    - noms : N2 et N3.
    - N2 et N3 sont le nombre de cubes sur l'axe des Y et des Z, respectivement d'une bote englobante d'une structure S.
      Ils servent pour la formule de conversion d'indices de tableau  3 dimensions vers tableau  1 dimension.

  Argument 3 :
    - type : integer_t (ici alias de unsigned short).
    - nom : N1xN2xN3.
    - N1xN2xN3 est le nombre total de cubes dans la bote englobante  de S.

  Argument 4 :integer_t (ici alias de unsigned short).
    - type : rfrence sur MatrixT (ici alias de std::vector<std::vector<bool> >).
    - nom : somaMatrix.
    - somaMatrix est la matrice boolenne dont les lignes forment l'ensemble de toutes les positions possibles
      de toutes les pices du jeu SOMA dans S.

  Argument 5 :
    - type : integer_t (ici alias de unsigned short).
    - nom : pieceIndex.
    - pieceIndex est l'indice de la pice pour laquelle computeAllSomaPositions a calcul une position.
      pieceIndex vaut 0 pour A, 1 pour B, 2 pour L, 3 pour P, 4 pour T et 6 pour Z.

  Arguments 6  17 :
    - type : integer_t (ici alias de unsigned short).
    - noms : i1, j1, k1, i2, j2, k2, i3, j3, k3, i4, j4 et k4.
    - Les 4 3-upls (i1,j1,k1), (i2, j2 ,k2), (i3,j3,k3) et (i4,j4,k4) correspondent aux positions de
      4 cubes diffrents dans la bote englobante de S.

  Retour : void.
*/
      template
        <
        typename IntegerT1,
        typename IntegerT2,
        typename MatrixT
        >
      inline void CAllSomaPiecesPositions::addNewPosToPieceList
                                                              (
                                                              const IntegerT1 N2,
                                                              const IntegerT1 N3,
                                                              const IntegerT2 N1xN2xN3,
                                                              MatrixT& somaMatrix,
                                                              const IntegerT1 pieceIndex,
                                                              const IntegerT1 i1,
                                                              const IntegerT1 j1,
                                                              const IntegerT1 k1,
                                                              const IntegerT1 i2,
                                                              const IntegerT1 j2,
                                                              const IntegerT1 k2,
                                                              const IntegerT1 i3,
                                                              const IntegerT1 j3,
                                                              const IntegerT1 k3,
                                                              const IntegerT1 i4,
                                                              const IntegerT1 j4,
                                                              const IntegerT1 k4
                                                              )
        {

// On cre une nouvelle ligne newRow comportant NUMBER_OF_PIECES+N1xN2xN3 colonnes.
        typename MatrixT::value_type newRow
                                          (
                                          NUMBER_OF_PIECES+N1xN2xN3,
                                          false
                                          );
// La colonne de newRow indique par pieceIndex (0, 1, 2, 3, 4 ou 6), ainsi que les 4 colonnes indiques par les 4
// 3-upls (i1,j1,k1), (i2, j2 ,k2), (i3,j3,k3) et (i4,j4,k4) sont assignes  1, toutes les autres tant assignes  0.
        newRow[pieceIndex]=
        newRow[positions_matrix_t::size_type(NUMBER_OF_PIECES+N3*(N2*i1+j1)+k1)]=
        newRow[positions_matrix_t::size_type(NUMBER_OF_PIECES+N3*(N2*i2+j2)+k2)]=
        newRow[positions_matrix_t::size_type(NUMBER_OF_PIECES+N3*(N2*i3+j3)+k3)]=
        newRow[positions_matrix_t::size_type(NUMBER_OF_PIECES+N3*(N2*i4+j4)+k4)]=true;
// La ligne newRow est ajoute  la suite des autres lignes de la matrice boolenne somaMatrix.
        somaMatrix.push_back
                           (
                           newRow
                           );

        }



/*
  La fonction membre prive computeAllPiecesPositions est d'une importance capitale car c'est elle qui calcule les positions possibles de chaque pice
  dans la structure  rsoudre, S. Auparavant, elle vrifie d'abord que les coordonnes des cubes que l'on a enregistrs ne sont pas indentiques auquel
  cas S ne serait pas constitue de 27 cubes donc elle n'aurait pas de solution. Elle calcule galement la parit de S. Si celle-ci est diffrente de -5,
  -3, -1, 1, 3 et 5, la fonction computeAllPiecesPositions ne calcule aucune position car S n'a pas de solution.
  Si la fonction computeAllPiecesPositions n'a pas dtermin que S n'avait pas de solutions, elle calcule toutes les positions possibles de chaque pice dans S.
  Les positions des cubes sont alors donnes relativement  la bote englobante de S. Il y a 12 faons diffrentes de tourner les pices A et B (qui sont symtriques),
  T, V et Z. Il n'y a que 8 manires diffrentes de tourner la pice P. En revanche, la pice L peut tre tourne de 24 manires diffrentes.
  Grce  la parit, on rduit les listes des positions possibles pour les pices V (toujours), P et T (quand S est de parit -5, -3, 3 ou 5 pour ces 2 pices).

  Argument : Aucun.

  Retour : void
*/
      inline void CAllSomaPiecesPositions::computeAllPiecesPositions()
        {

        const unsigned char N1=(unsigned char)m_BoundingBox.size1(),
                            N2=(unsigned char)m_BoundingBox.size2(),
                            N3=(unsigned char)m_BoundingBox.size3();

        const bit_array_t::size_type N1xN2xN3=m_BoundingBox.size();

//  On colore, alternativement, avec 2 couleus (disons rouge et vert), les cubes de la bote englobante d'une structure S.
//  Pour ce faire, on dclare un tableau de boolens tri_dimensionnel aux mmes dimensions que la bote englobante de S,
//  dans lequel on alterne les 1 et les 0.
        const bit_array_t chessBoard3DForParityCheck=bit_array_t::generate3DChessBoard
                                                                                     (
                                                                                     N1,
                                                                                     N2,
                                                                                     N3
                                                                                     );

        unsigned char nbDifferentCubelets=0,
                      nbRedsInStructure=0;
        bit_array_t::const_iterator boundingBoxIter=m_BoundingBox.begin(),
                                    chessBoardIter=chessBoard3DForParityCheck.begin();
        for(bit_array_t::size_type i=0;i<N1xN2xN3;++i)
          {

//  On compte le nombre de cubes aux coordonnes rellement diffrentes dans S. Il doit y en avoir exactement 27.
          nbDifferentCubelets+=(unsigned char)(boundingBoxIter[i]);

//  On compte le nombre de cubes colors en rouge dans S. Pour ce faire, on compare les 1 de la bote englobante
//  de S (qui modlisent une position de cube) et les de la bote englobante (qui modlisent une couleur, disons rouge).
//  Ds qu'il y a 2 1 en mme temps, on dit que le nombre de cubes rouges dans S est augment de 1.
          nbRedsInStructure+=(unsigned char)(boundingBoxIter[i]&chessBoardIter[i]);

          }

//  Soit nR, le nombre de cubes rouges dans S et nV celui de verts. Comme S est constitue de 27 cubes, on a : nR + nV = 27.
//  Donc nV = 27 - nR
//  Soit p la parit de S, par dfinition, p = nR - nV.
//  Donc p = nR - (27 - nR) = nR - 27 +nR = 2 x nR - 27.
//  Donc, on a seulement besoin de connatre le nombre de cubes rouges dans S pour dterminer sa parit; d'o le calcul suivant :
        const char structureParity=2*nbRedsInStructure-27;

        const unsigned char absStructureParity=std::abs
                                                      (
                                                      structureParity
                                                      );

//  Si S a pour parit une valeur diffrente de -5, -3, -1, 1, 3 ou 5 (ou de manire quivalente si la valeur absolue de la
//  parit de S est diffrente de 1, 3 ou 5), alors la structure n'a pas de solution et m_HasNoSolution est assigne  true.
//  Du coup, il est inutile de calculer quelque position de pice que ce soit.
        if(!(m_HasNoSolution=((nbDifferentCubelets!=27)||((absStructureParity!=1)&&(absStructureParity!=3)&&(absStructureParity!=5)))))
          {

//  Les pices A, B, L et Z ont toujours pour parit 0.
//  P et T sont toutes 2 de parit -2 quand S est de parit -3 ou -5 et toutes 2 de parit 2 quand S a pour parit 3 ou 5.
//  En revanche, quand la parit de S vaut -1 ou 1, on ne peut rien dire car l'une est de parit 2 et l'autre -2 sans qu'on
//  ne sache laquelle.
          const char piecesPAndTParity=((structureParity==3)||(structureParity==5))?2:-2,
//  Si S a une parit positive (1, 3 ou 5) alors V a pour parit 1. Sinon (S a pour parit -5, -3 ou -1) la parit de V est -1.
                     pieceVParity=((structureParity==-3)||(structureParity==1)||(structureParity==5))?1:-1;


          unsigned char i=0,
                        iPlus1=1,
                        iPlus2=2,
                        j=0,
                        jPlus1=1,
                        jPlus2=2,
                        k=0,
                        kPlus1=1,
                        kPlus2=2;


          for(;i<N1;++i)
            {

            iPlus2=(iPlus1=i+1)+1;

            for(j=0;j<N2;++j)
              {

              jPlus2=(jPlus1=j+1)+1;

              for(k=0;k<N3;++k)
                {

                kPlus2=(kPlus1=k+1)+1;

                if((iPlus1<N1)&&(jPlus1<N2)&&(kPlus1<N3))
                  {

// Dbut du calcul des positions de A dans la structure  rsoudre.

                  if(m_BoundingBox(i,j,k)&m_BoundingBox(iPlus1,j,k)&m_BoundingBox(i,j,kPlus1)&m_BoundingBox(i,jPlus1,kPlus1))
                    addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)0),i,j,k,iPlus1,j,k,i,j,kPlus1,i,jPlus1,kPlus1);

                  if(m_BoundingBox(i,j,k)&m_BoundingBox(i,j,kPlus1)&m_BoundingBox(i,jPlus1,k)&m_BoundingBox(iPlus1,jPlus1,k))
                    addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)0),i,j,k,i,j,kPlus1,i,jPlus1,k,iPlus1,jPlus1,k);

                  if(m_BoundingBox(i,j,k)&m_BoundingBox(i,jPlus1,k)&m_BoundingBox(i,jPlus1,kPlus1)&m_BoundingBox(iPlus1,jPlus1,kPlus1))
                    addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)0),i,j,k,i,jPlus1,k,i,jPlus1,kPlus1,iPlus1,jPlus1,kPlus1);

                  if(m_BoundingBox(i,j,k)&m_BoundingBox(iPlus1,j,k)&m_BoundingBox(iPlus1,jPlus1,k)&m_BoundingBox(iPlus1,jPlus1,kPlus1))
                    addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)0),i,j,k,iPlus1,j,k,iPlus1,jPlus1,k,iPlus1,jPlus1,kPlus1);

                  if(m_BoundingBox(i,j,k)&m_BoundingBox(i,j,kPlus1)&m_BoundingBox(iPlus1,j,kPlus1)&m_BoundingBox(iPlus1,jPlus1,kPlus1))
                    addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)0),i,j,k,i,j,kPlus1,iPlus1,j,kPlus1,iPlus1,jPlus1,kPlus1);

                  if(m_BoundingBox(i,j,k)&m_BoundingBox(iPlus1,j,k)&m_BoundingBox(iPlus1,j,kPlus1)&m_BoundingBox(i,jPlus1,k))
                    addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)0),i,j,k,iPlus1,j,k,iPlus1,j,kPlus1,i,jPlus1,k);

                  if(m_BoundingBox(i,j,kPlus1)&m_BoundingBox(iPlus1,j,kPlus1)&m_BoundingBox(i,jPlus1,k)&m_BoundingBox(i,jPlus1,kPlus1))
                    addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)0),i,j,kPlus1,iPlus1,j,kPlus1,i,jPlus1,k,i,jPlus1,kPlus1);

                  if(m_BoundingBox(i,j,kPlus1)&m_BoundingBox(iPlus1,jPlus1,k)&m_BoundingBox(i,jPlus1,kPlus1)&m_BoundingBox(iPlus1,jPlus1,kPlus1))
                    addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)0),i,j,kPlus1,iPlus1,jPlus1,k,i,jPlus1,kPlus1,iPlus1,jPlus1,kPlus1);

                  if(m_BoundingBox(iPlus1,j,k)&m_BoundingBox(iPlus1,j,kPlus1)&m_BoundingBox(i,jPlus1,kPlus1)&m_BoundingBox(iPlus1,jPlus1,kPlus1))
                    addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)0),iPlus1,j,k,iPlus1,j,kPlus1,i,jPlus1,kPlus1,iPlus1,jPlus1,kPlus1);

                  if(m_BoundingBox(iPlus1,j,k)&m_BoundingBox(i,j,kPlus1)&m_BoundingBox(iPlus1,j,kPlus1)&m_BoundingBox(iPlus1,jPlus1,k))
                    addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)0),iPlus1,j,k,i,j,kPlus1,iPlus1,j,kPlus1,iPlus1,jPlus1,k);

                  if(m_BoundingBox(iPlus1,j,kPlus1)&m_BoundingBox(i,jPlus1,k)&m_BoundingBox(iPlus1,jPlus1,k)&m_BoundingBox(iPlus1,jPlus1,kPlus1))
                    addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)0),iPlus1,j,kPlus1,i,jPlus1,k,iPlus1,jPlus1,k,iPlus1,jPlus1,kPlus1);

                  if(m_BoundingBox(iPlus1,j,k)&m_BoundingBox(i,jPlus1,k)&m_BoundingBox(iPlus1,jPlus1,k)&m_BoundingBox(i,jPlus1,kPlus1))
                    addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)0),iPlus1,j,k,i,jPlus1,k,iPlus1,jPlus1,k,i,jPlus1,kPlus1);

// Fin du calcul des positions de A dans la structure  rsoudre.

// Dbut du calcul des positions de B dans la structure  rsoudre.

                  if(m_BoundingBox(i,j,k)&m_BoundingBox(i,j,kPlus1)&m_BoundingBox(i,jPlus1,kPlus1)&m_BoundingBox(iPlus1,jPlus1,kPlus1))
                    addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)1),i,j,k,i,j,kPlus1,i,jPlus1,kPlus1,iPlus1,jPlus1,kPlus1);

                  if(m_BoundingBox(i,j,k)&m_BoundingBox(i,j,kPlus1)&m_BoundingBox(iPlus1,j,kPlus1)&m_BoundingBox(i,jPlus1,k))
                    addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)1),i,j,k,i,j,kPlus1,iPlus1,j,kPlus1,i,jPlus1,k);

                  if(m_BoundingBox(i,j,k)&m_BoundingBox(iPlus1,j,k)&m_BoundingBox(i,jPlus1,k)&m_BoundingBox(i,jPlus1,kPlus1))
                    addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)1),i,j,k,iPlus1,j,k,i,jPlus1,k,i,jPlus1,kPlus1);

                  if(m_BoundingBox(i,j,k)&m_BoundingBox(iPlus1,j,k)&m_BoundingBox(iPlus1,j,kPlus1)&m_BoundingBox(iPlus1,jPlus1,kPlus1))
                    addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)1),i,j,k,iPlus1,j,k,iPlus1,j,kPlus1,iPlus1,jPlus1,kPlus1);

                  if(m_BoundingBox(i,j,k)&m_BoundingBox(iPlus1,j,k)&m_BoundingBox(i,j,kPlus1)&m_BoundingBox(iPlus1,jPlus1,k))
                    addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)1),i,j,k,iPlus1,j,k,i,j,kPlus1,iPlus1,jPlus1,k);

                  if(m_BoundingBox(i,j,k)&m_BoundingBox(i,jPlus1,k)&m_BoundingBox(iPlus1,jPlus1,k)&m_BoundingBox(iPlus1,jPlus1,kPlus1))
                    addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)1),i,j,k,i,jPlus1,k,iPlus1,jPlus1,k,iPlus1,jPlus1,kPlus1);

                  if(m_BoundingBox(i,j,kPlus1)&m_BoundingBox(i,jPlus1,k)&m_BoundingBox(iPlus1,jPlus1,k)&m_BoundingBox(i,jPlus1,kPlus1))
                    addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)1),i,j,kPlus1,i,jPlus1,k,iPlus1,jPlus1,k,i,jPlus1,kPlus1);

                  if(m_BoundingBox(i,j,kPlus1)&m_BoundingBox(iPlus1,j,kPlus1)&m_BoundingBox(iPlus1,jPlus1,k)&m_BoundingBox(iPlus1,jPlus1,kPlus1))
                    addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)1),i,j,kPlus1,iPlus1,j,kPlus1,iPlus1,jPlus1,k,iPlus1,jPlus1,kPlus1);

                  if(m_BoundingBox(iPlus1,j,k)&m_BoundingBox(iPlus1,j,kPlus1)&m_BoundingBox(i,jPlus1,k)&m_BoundingBox(iPlus1,jPlus1,k))
                    addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)1),iPlus1,j,k,iPlus1,j,kPlus1,i,jPlus1,k,iPlus1,jPlus1,k);

                  if(m_BoundingBox(iPlus1,j,k)&m_BoundingBox(iPlus1,jPlus1,k)&m_BoundingBox(i,jPlus1,kPlus1)&m_BoundingBox(iPlus1,jPlus1,kPlus1))
                    addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)1),iPlus1,j,k,iPlus1,jPlus1,k,i,jPlus1,kPlus1,iPlus1,jPlus1,kPlus1);

                  if(m_BoundingBox(iPlus1,j,k)&m_BoundingBox(iPlus1,j,kPlus1)&m_BoundingBox(i,j,kPlus1)&m_BoundingBox(i,jPlus1,kPlus1))
                    addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)1),iPlus1,j,k,iPlus1,j,kPlus1,i,j,kPlus1,i,jPlus1,kPlus1);

                  if(m_BoundingBox(iPlus1,j,kPlus1)&m_BoundingBox(i,jPlus1,k)&m_BoundingBox(i,jPlus1,kPlus1)&m_BoundingBox(iPlus1,jPlus1,kPlus1))
                    addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)1),iPlus1,j,kPlus1,i,jPlus1,k,i,jPlus1,kPlus1,iPlus1,jPlus1,kPlus1);

// Fin du calcul des positions de B dans la structure  rsoudre.

// Dbut du calcul des positions de P dans la structure  rsoudre.

//  Soit nR, le nombre de cubes rouges dans la pice P et nV celui de verts. Comme P possde de 4 cubes, on a : nR + nV = 4.
//  Donc nV = 4 - nR
//  Soit pP la parit de P, par dfinition, pP = nR - nV.
//  Donc pP = nR - (4 - nR) = nR - 4 + nR = 2 x nR - 4.
//  Donc, si S est de parit diffrente de -1 ou 1; Donc il suffit de comparer pP  2 ou -2 (selon la parit de S)
//  pour savoir si on enregistre ou pas la position calcule (mme si elle est valide).

                  if(
                     (m_BoundingBox(i,j,k)&m_BoundingBox(i,j,kPlus1)&m_BoundingBox(iPlus1,j,kPlus1)&m_BoundingBox(i,jPlus1,kPlus1))&&
                     (
                      (absStructureParity!=1)?
                       (
                        (
                         2*(
                            (char)(chessBoard3DForParityCheck(i,j,k))+
                            (char)(chessBoard3DForParityCheck(i,j,kPlus1))+
                            (char)(chessBoard3DForParityCheck(iPlus1,j,kPlus1))+
                            (char)(chessBoard3DForParityCheck(i,jPlus1,kPlus1))
                           )-4
                        )==piecesPAndTParity
                       ) : true
                     )
                    )
                    addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)3),i,j,k,i,j,kPlus1,iPlus1,j,kPlus1,i,jPlus1,kPlus1);

                  if(
                     (m_BoundingBox(i,j,k)&m_BoundingBox(iPlus1,j,k)&m_BoundingBox(i,j,kPlus1)&m_BoundingBox(i,jPlus1,k))&&
                     (
                      (absStructureParity!=1)?
                       (
                        (
                         2*(
                            (char)(chessBoard3DForParityCheck(i,j,k))+
                            (char)(chessBoard3DForParityCheck(iPlus1,j,k))+
                            (char)(chessBoard3DForParityCheck(i,j,kPlus1))+
                            (char)(chessBoard3DForParityCheck(i,jPlus1,k))
                           )-4
                        )==piecesPAndTParity
                       ) : true
                     )
                    )
                    addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)3),i,j,k,iPlus1,j,k,i,j,kPlus1,i,jPlus1,k);

                  if(
                     (m_BoundingBox(i,j,k)&m_BoundingBox(i,jPlus1,k)&m_BoundingBox(iPlus1,jPlus1,k)&m_BoundingBox(i,jPlus1,kPlus1))&&
                     (
                      (absStructureParity!=1)?
                       (
                        (
                         2*(
                            (char)(chessBoard3DForParityCheck(i,j,k))+
                            (char)(chessBoard3DForParityCheck(i,jPlus1,k))+
                            (char)(chessBoard3DForParityCheck(iPlus1,jPlus1,k))+
                            (char)(chessBoard3DForParityCheck(i,jPlus1,kPlus1))
                           )-4
                        )==piecesPAndTParity
                       ) : true
                     )
                    )
                    addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)3),i,j,k,i,jPlus1,k,iPlus1,jPlus1,k,i,jPlus1,kPlus1);

                  if(
                     (m_BoundingBox(i,j,k)&m_BoundingBox(iPlus1,j,k)&m_BoundingBox(iPlus1,j,kPlus1)&m_BoundingBox(iPlus1,jPlus1,k))&&
                     (
                      (absStructureParity!=1)?
                       (
                        (
                         2*(
                            (char)(chessBoard3DForParityCheck(i,j,k))+
                            (char)(chessBoard3DForParityCheck(iPlus1,j,k))+
                            (char)(chessBoard3DForParityCheck(iPlus1,j,kPlus1))+
                            (char)(chessBoard3DForParityCheck(iPlus1,jPlus1,k))
                           )-4
                        )==piecesPAndTParity
                       ) : true
                     )
                    )
                    addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)3),i,j,k,iPlus1,j,k,iPlus1,j,kPlus1,iPlus1,jPlus1,k);

                  if(
                     (m_BoundingBox(i,j,kPlus1)&m_BoundingBox(i,jPlus1,k)&m_BoundingBox(i,jPlus1,kPlus1)&m_BoundingBox(iPlus1,jPlus1,kPlus1))&&
                     (
                      (absStructureParity!=1)?
                       (
                        (
                         2*(
                            (char)(chessBoard3DForParityCheck(i,j,kPlus1))+
                            (char)(chessBoard3DForParityCheck(i,jPlus1,k))+
                            (char)(chessBoard3DForParityCheck(i,jPlus1,kPlus1))+
                            (char)(chessBoard3DForParityCheck(iPlus1,jPlus1,kPlus1))
                           )-4
                        )==piecesPAndTParity
                       ) : true
                     )
                    )
                    addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)3),i,j,kPlus1,i,jPlus1,k,i,jPlus1,kPlus1,iPlus1,jPlus1,kPlus1);

                  if(
                     (m_BoundingBox(iPlus1,j,k)&m_BoundingBox(i,j,kPlus1)&m_BoundingBox(iPlus1,j,kPlus1)&m_BoundingBox(iPlus1,jPlus1,kPlus1))&&
                     (
                      (absStructureParity!=1)?
                       (
                        (
                         2*(
                            (char)(chessBoard3DForParityCheck(iPlus1,j,k))+
                            (char)(chessBoard3DForParityCheck(i,j,kPlus1))+
                            (char)(chessBoard3DForParityCheck(iPlus1,j,kPlus1))+
                            (char)(chessBoard3DForParityCheck(iPlus1,jPlus1,kPlus1))
                           )-4
                        )==piecesPAndTParity
                       ) : true
                     )
                    )
                    addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)3),iPlus1,j,k,i,j,kPlus1,iPlus1,j,kPlus1,iPlus1,jPlus1,kPlus1);

                  if(
                     (m_BoundingBox(iPlus1,j,k)&m_BoundingBox(i,jPlus1,k)&m_BoundingBox(iPlus1,jPlus1,k)&m_BoundingBox(iPlus1,jPlus1,kPlus1))&&
                     (
                      (absStructureParity!=1)?
                       (
                        (
                         2*(
                            (char)(chessBoard3DForParityCheck(iPlus1,j,k))+
                            (char)(chessBoard3DForParityCheck(i,jPlus1,k))+
                            (char)(chessBoard3DForParityCheck(iPlus1,jPlus1,k))+
                            (char)(chessBoard3DForParityCheck(iPlus1,jPlus1,kPlus1))
                           )-4
                        )==piecesPAndTParity
                       ) : true
                     )
                    )
                    addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)3),iPlus1,j,k,i,jPlus1,k,iPlus1,jPlus1,k,iPlus1,jPlus1,kPlus1);

                  if(
                     (m_BoundingBox(iPlus1,j,kPlus1)&m_BoundingBox(iPlus1,jPlus1,k)&m_BoundingBox(i,jPlus1,kPlus1)&m_BoundingBox(iPlus1,jPlus1,kPlus1))&&
                     (
                      (absStructureParity!=1)?
                       (
                        (
                         2*(
                            (char)(chessBoard3DForParityCheck(iPlus1,j,kPlus1))+
                            (char)(chessBoard3DForParityCheck(iPlus1,jPlus1,k))+
                            (char)(chessBoard3DForParityCheck(i,jPlus1,kPlus1))+
                            (char)(chessBoard3DForParityCheck(iPlus1,jPlus1,kPlus1))
                           )-4
                        )==piecesPAndTParity
                       ) : true
                     )
                    )
                    addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)3),iPlus1,j,kPlus1,iPlus1,jPlus1,k,i,jPlus1,kPlus1,iPlus1,jPlus1,kPlus1);

// Fin du calcul des positions de P dans la structure  rsoudre.

                  }

                if((iPlus1<N1)&&(jPlus1<N2))
                  {

                  if(iPlus2<N1)
                    {

// Dbut du calcul des positions de L dans la structure  rsoudre.

                    if(m_BoundingBox(i,j,k)&m_BoundingBox(iPlus1,j,k)&m_BoundingBox(iPlus2,j,k)&m_BoundingBox(iPlus2,jPlus1,k))
                      addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)2),i,j,k,iPlus1,j,k,iPlus2,j,k,iPlus2,jPlus1,k);

                    if(m_BoundingBox(i,j,k)&m_BoundingBox(iPlus1,j,k)&m_BoundingBox(iPlus2,j,k)&m_BoundingBox(i,jPlus1,k))
                      addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)2),i,j,k,iPlus1,j,k,iPlus2,j,k,i,jPlus1,k);

                    if(m_BoundingBox(i,j,k)&m_BoundingBox(i,jPlus1,k)&m_BoundingBox(iPlus1,jPlus1,k)&m_BoundingBox(iPlus2,jPlus1,k))
                      addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)2),i,j,k,i,jPlus1,k,iPlus1,jPlus1,k,iPlus2,jPlus1,k);

                    if(m_BoundingBox(iPlus2,j,k)&m_BoundingBox(i,jPlus1,k)&m_BoundingBox(iPlus1,jPlus1,k)&m_BoundingBox(iPlus2,jPlus1,k))
                      addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)2),iPlus2,j,k,i,jPlus1,k,iPlus1,jPlus1,k,iPlus2,jPlus1,k);

// Fin du calcul des positions de L dans la structure  rsoudre.

// Dbut du calcul des positions de T dans la structure  rsoudre.

//  Soit nR, le nombre de cubes rouges dans la pice T et nV celui de verts. Comme T possde de 4 cubes, on a : nR + nV = 4.
//  Donc nV = 4 - nR
//  Soit pT la parit de T, par dfinition, pP = nR - nV.
//  Donc pT = nR - (4 - nR) = nR - 4 + nR = 2 x nR - 4.
//  Donc, si S est de parit diffrente de -1 ou 1; Donc il suffit de comparer pT  2 ou -2 (selon la parit de S)
//  pour savoir si on enregistre ou pas la position calcule (mme si elle est valide).

                    if(
                       (m_BoundingBox(i,j,k)&m_BoundingBox(iPlus1,j,k)&m_BoundingBox(iPlus1,jPlus1,k)&m_BoundingBox(iPlus2,j,k))&&
                       (
                        (absStructureParity!=1)?
                         (
                          (
                           2*(
                              (char)(chessBoard3DForParityCheck(i,j,k))+
                              (char)(chessBoard3DForParityCheck(iPlus1,j,k))+
                              (char)(chessBoard3DForParityCheck(iPlus1,jPlus1,k))+
                              (char)(chessBoard3DForParityCheck(iPlus2,j,k))
                             )-4
                          )==piecesPAndTParity
                         ) : true
                       )
                      )
                      addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)4),i,j,k,iPlus1,j,k,iPlus1,jPlus1,k,iPlus2,j,k);

                    if(
                       (m_BoundingBox(iPlus1,j,k)&m_BoundingBox(i,jPlus1,k)&m_BoundingBox(iPlus1,jPlus1,k)&m_BoundingBox(iPlus2,jPlus1,k))&&
                       (
                        (absStructureParity!=1)?
                         (
                          (
                           2*(
                              (char)(chessBoard3DForParityCheck(iPlus1,j,k))+
                              (char)(chessBoard3DForParityCheck(i,jPlus1,k))+
                              (char)(chessBoard3DForParityCheck(iPlus1,jPlus1,k))+
                              (char)(chessBoard3DForParityCheck(iPlus2,jPlus1,k))
                             )-4
                          )==piecesPAndTParity
                         ) : true
                       )
                      )
                      addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)4),iPlus1,j,k,i,jPlus1,k,iPlus1,jPlus1,k,iPlus2,jPlus1,k);

// Fin du calcul des positions de T dans la structure  rsoudre.

// Dbut du calcul des positions de Z dans la structure  rsoudre.

                    if(m_BoundingBox(i,j,k)&m_BoundingBox(iPlus1,j,k)&m_BoundingBox(iPlus1,jPlus1,k)&m_BoundingBox(iPlus2,jPlus1,k))
                      addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)6),i,j,k,iPlus1,j,k,iPlus1,jPlus1,k,iPlus2,jPlus1,k);

                    if(m_BoundingBox(iPlus2,j,k)&m_BoundingBox(iPlus1,j,k)&m_BoundingBox(i,jPlus1,k)&m_BoundingBox(iPlus1,jPlus1,k))
                      addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)6),iPlus2,j,k,iPlus1,j,k,i,jPlus1,k,iPlus1,jPlus1,k);

// Fin du calcul des positions de Z dans la structure  rsoudre.

                    }

                  if(jPlus2<N2)
                    {

// Dbut du calcul des positions de L dans la structure  rsoudre.

                    if(m_BoundingBox(i,j,k)&m_BoundingBox(iPlus1,j,k)&m_BoundingBox(i,jPlus1,k)&m_BoundingBox(i,jPlus2,k))
                      addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)2),i,j,k,iPlus1,j,k,i,jPlus1,k,i,jPlus2,k);

                    if(m_BoundingBox(i,j,k)&m_BoundingBox(i,jPlus1,k)&m_BoundingBox(i,jPlus2,k)&m_BoundingBox(iPlus1,jPlus2,k))
                      addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)2),i,j,k,i,jPlus1,k,i,jPlus2,k,iPlus1,jPlus2,k);

                    if(m_BoundingBox(i,j,k)&m_BoundingBox(iPlus1,j,k)&m_BoundingBox(iPlus1,jPlus1,k)&m_BoundingBox(iPlus1,jPlus2,k))
                      addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)2),i,j,k,iPlus1,j,k,iPlus1,jPlus1,k,iPlus1,jPlus2,k);

                    if(m_BoundingBox(iPlus1,j,k)&m_BoundingBox(iPlus1,jPlus1,k)&m_BoundingBox(i,jPlus2,k)&m_BoundingBox(iPlus1,jPlus2,k))
                      addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)2),iPlus1,j,k,iPlus1,jPlus1,k,i,jPlus2,k,iPlus1,jPlus2,k);

// Fin du calcul des positions de L dans la structure  rsoudre.

// Dbut du calcul des positions de T dans la structure  rsoudre.

                    if(
                       (m_BoundingBox(i,j,k)&m_BoundingBox(i,jPlus1,k)&m_BoundingBox(iPlus1,jPlus1,k)&m_BoundingBox(i,jPlus2,k))&&
                       (
                        (absStructureParity!=1)?
                         (
                          (
                           2*(
                              (char)(chessBoard3DForParityCheck(i,j,k))+
                              (char)(chessBoard3DForParityCheck(i,jPlus1,k))+
                              (char)(chessBoard3DForParityCheck(iPlus1,jPlus1,k))+
                              (char)(chessBoard3DForParityCheck(i,jPlus2,k))
                             )-4
                          )==piecesPAndTParity
                         ) : true
                       )
                      )
                      addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)4),i,j,k,i,jPlus1,k,iPlus1,jPlus1,k,i,jPlus2,k);

                    if(
                       (m_BoundingBox(iPlus1,j,k)&m_BoundingBox(i,jPlus1,k)&m_BoundingBox(iPlus1,jPlus1,k)&m_BoundingBox(iPlus1,jPlus2,k))&&
                       (
                        (absStructureParity!=1)?
                         (
                          (
                           2*(
                              (char)(chessBoard3DForParityCheck(iPlus1,j,k))+
                              (char)(chessBoard3DForParityCheck(i,jPlus1,k))+
                              (char)(chessBoard3DForParityCheck(iPlus1,jPlus1,k))+
                              (char)(chessBoard3DForParityCheck(iPlus1,jPlus2,k))
                             )-4
                          )==piecesPAndTParity
                         ) : true
                       )
                      )
                      addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)4),iPlus1,j,k,i,jPlus1,k,iPlus1,jPlus1,k,iPlus1,jPlus2,k);

// Fin du calcul des positions de T dans la structure  rsoudre.

// Dbut du calcul des positions de Z dans la structure  rsoudre.

                    if(m_BoundingBox(i,j,k)&m_BoundingBox(i,jPlus1,k)&m_BoundingBox(iPlus1,jPlus1,k)&m_BoundingBox(iPlus1,jPlus2,k))
                      addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)6),i,j,k,i,jPlus1,k,iPlus1,jPlus1,k,iPlus1,jPlus2,k);

                    if(m_BoundingBox(iPlus1,j,k)&m_BoundingBox(i,jPlus1,k)&m_BoundingBox(iPlus1,jPlus1,k)&m_BoundingBox(i,jPlus2,k))
                      addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)6),iPlus1,j,k,i,jPlus1,k,iPlus1,jPlus1,k,i,jPlus2,k);

// Fin du calcul des positions de Z dans la structure  rsoudre.

                    }

// Dbut du calcul des positions de V dans la structure  rsoudre.

//  Soit nR, le nombre de cubes rouges dans la pice V et nV celui de verts. Comme V possde de 3 cubes, on a : nR + nV = 3.
//  Donc nV = 3 - nR
//  Soit pV la parit de V, par dfinition, pV = nR - nV.
//  Donc pV = nR - (3 - nR) = nR - 3 + nR = 2 x nR - 3.
//  Donc il suffit de comparer pV  1 ou -1 (selon la parit de S) pour savoir si on enregistre ou pas la position calcule (mme si elle est valide).

                  if(
                     (m_BoundingBox(i,j,k)&m_BoundingBox(iPlus1,j,k)&m_BoundingBox(i,jPlus1,k))&&
                     (
                      (
                       2*(
                          (char)(chessBoard3DForParityCheck(i,j,k))+
                          (char)(chessBoard3DForParityCheck(iPlus1,j,k))+
                          (char)(chessBoard3DForParityCheck(i,jPlus1,k))
                         )-3
                      )==pieceVParity
                     )
                    )
                    addNewPosToVList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,i,j,k,iPlus1,j,k,i,jPlus1,k);

                  if(
                     (m_BoundingBox(i,j,k)&m_BoundingBox(i,jPlus1,k)&m_BoundingBox(iPlus1,jPlus1,k))&&
                     (
                      (
                       2*(
                          (char)(chessBoard3DForParityCheck(i,j,k))+
                          (char)(chessBoard3DForParityCheck(i,jPlus1,k))+
                          (char)(chessBoard3DForParityCheck(iPlus1,jPlus1,k))
                         )-3
                      )==pieceVParity
                     )
                    )
                    addNewPosToVList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,i,j,k,i,jPlus1,k,iPlus1,jPlus1,k);

                  if(
                     (m_BoundingBox(i,j,k)&m_BoundingBox(iPlus1,j,k)&m_BoundingBox(iPlus1,jPlus1,k))&&
                     (
                      (
                       2*(
                          (char)(chessBoard3DForParityCheck(i,j,k))+
                          (char)(chessBoard3DForParityCheck(iPlus1,j,k))+
                          (char)(chessBoard3DForParityCheck(iPlus1,jPlus1,k))
                         )-3
                      )==pieceVParity
                     )
                    )
                    addNewPosToVList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,i,j,k,iPlus1,j,k,iPlus1,jPlus1,k);

                  if(
                     (m_BoundingBox(iPlus1,j,k)&m_BoundingBox(i,jPlus1,k)&m_BoundingBox(iPlus1,jPlus1,k))&&
                     (
                      (
                       2*(
                          (char)(chessBoard3DForParityCheck(iPlus1,j,k))+
                          (char)(chessBoard3DForParityCheck(i,jPlus1,k))+
                          (char)(chessBoard3DForParityCheck(iPlus1,jPlus1,k))
                         )-3
                      )==pieceVParity
                     )
                    )
                    addNewPosToVList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,iPlus1,j,k,i,jPlus1,k,iPlus1,jPlus1,k);

// Fin du calcul des positions de V dans la structure  rsoudre.

                  }

                if((iPlus1<N1)&&(kPlus1<N3))
                  {

                  if(iPlus2<N1)
                    {

// Dbut du calcul des positions de L dans la structure  rsoudre.

                    if(m_BoundingBox(i,j,k)&m_BoundingBox(iPlus1,j,k)&m_BoundingBox(iPlus2,j,k)&m_BoundingBox(iPlus2,j,kPlus1))
                      addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)2),i,j,k,iPlus1,j,k,iPlus2,j,k,iPlus2,j,kPlus1);

                    if(m_BoundingBox(i,j,k)&m_BoundingBox(iPlus1,j,k)&m_BoundingBox(iPlus2,j,k)&m_BoundingBox(i,j,kPlus1))
                      addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)2),i,j,k,iPlus1,j,k,iPlus2,j,k,i,j,kPlus1);

                    if(m_BoundingBox(i,j,k)&m_BoundingBox(i,j,kPlus1)&m_BoundingBox(iPlus1,j,kPlus1)&m_BoundingBox(iPlus2,j,kPlus1))
                      addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)2),i,j,k,i,j,kPlus1,iPlus1,j,kPlus1,iPlus2,j,kPlus1);

                    if(m_BoundingBox(iPlus2,j,k)&m_BoundingBox(i,j,kPlus1)&m_BoundingBox(iPlus1,j,kPlus1)&m_BoundingBox(iPlus2,j,kPlus1))
                      addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)2),iPlus2,j,k,i,j,kPlus1,iPlus1,j,kPlus1,iPlus2,j,kPlus1);

// Fin du calcul des positions de L dans la structure  rsoudre.

// Dbut du calcul des positions de T dans la structure  rsoudre.

                    if(
                       (m_BoundingBox(i,j,k)&m_BoundingBox(iPlus1,j,k)&m_BoundingBox(iPlus1,j,kPlus1)&m_BoundingBox(iPlus2,j,k))&&
                       (
                        (absStructureParity!=1)?
                         (
                          (
                           2*(
                              (char)(chessBoard3DForParityCheck(i,j,k))+
                              (char)(chessBoard3DForParityCheck(iPlus1,j,k))+
                              (char)(chessBoard3DForParityCheck(iPlus1,j,kPlus1))+
                              (char)(chessBoard3DForParityCheck(iPlus2,j,k))
                             )-4
                          )==piecesPAndTParity
                         ) : true
                       )
                      )
                      addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)4),i,j,k,iPlus1,j,k,iPlus1,j,kPlus1,iPlus2,j,k);

                    if(
                       (m_BoundingBox(iPlus1,j,k)&m_BoundingBox(i,j,kPlus1)&m_BoundingBox(iPlus1,j,kPlus1)&m_BoundingBox(iPlus2,j,kPlus1))&&
                       (
                        (absStructureParity!=1)?
                         (
                          (
                           2*(
                              (char)(chessBoard3DForParityCheck(iPlus1,j,k))+
                              (char)(chessBoard3DForParityCheck(i,j,kPlus1))+
                              (char)(chessBoard3DForParityCheck(iPlus1,j,kPlus1))+
                              (char)(chessBoard3DForParityCheck(iPlus2,j,kPlus1))
                             )-4
                          )==piecesPAndTParity
                         ) : true
                       )
                      )
                      addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)4),iPlus1,j,k,i,j,kPlus1,iPlus1,j,kPlus1,iPlus2,j,kPlus1);

// Fin du calcul des positions de T dans la structure  rsoudre..

// Dbut du calcul des positions de Z dans la structure  rsoudre.

                    if(m_BoundingBox(i,j,k)&m_BoundingBox(iPlus1,j,k)&m_BoundingBox(iPlus1,j,kPlus1)&m_BoundingBox(iPlus2,j,kPlus1))
                      addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)6),i,j,k,iPlus1,j,k,iPlus1,j,kPlus1,iPlus2,j,kPlus1);

                    if(m_BoundingBox(iPlus2,j,k)&m_BoundingBox(iPlus1,j,k)&m_BoundingBox(i,j,kPlus1)&m_BoundingBox(iPlus1,j,kPlus1))
                      addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)6),iPlus2,j,k,iPlus1,j,k,i,j,kPlus1,iPlus1,j,kPlus1);

// Fin du calcul des positions de Z dans la structure  rsoudre.

                    }

                  if(kPlus2<N3)
                    {

// Dbut du calcul des positions de L dans la structure  rsoudre.

                    if(m_BoundingBox(i,j,k)&m_BoundingBox(i,j,kPlus1)&m_BoundingBox(i,j,kPlus2)&m_BoundingBox(iPlus1,j,kPlus2))
                      addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)2),i,j,k,i,j,kPlus1,i,j,kPlus2,iPlus1,j,kPlus2);

                    if(m_BoundingBox(i,j,k)&m_BoundingBox(iPlus1,j,k)&m_BoundingBox(i,j,kPlus1)&m_BoundingBox(i,j,kPlus2))
                      addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)2),i,j,k,iPlus1,j,k,i,j,kPlus1,i,j,kPlus2);

                    if(m_BoundingBox(i,j,k)&m_BoundingBox(iPlus1,j,k)&m_BoundingBox(iPlus1,j,kPlus1)&m_BoundingBox(iPlus1,j,kPlus2))
                      addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)2),i,j,k,iPlus1,j,k,iPlus1,j,kPlus1,iPlus1,j,kPlus2);

                    if(m_BoundingBox(iPlus1,j,k)&m_BoundingBox(iPlus1,j,kPlus1)&m_BoundingBox(i,j,kPlus2)&m_BoundingBox(iPlus1,j,kPlus2))
                      addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)2),iPlus1,j,k,iPlus1,j,kPlus1,i,j,kPlus2,iPlus1,j,kPlus2);

// Fin du calcul des positions de L dans la structure  rsoudre.

// Dbut du calcul des positions de T dans la structure  rsoudre.

                    if(
                       (m_BoundingBox(i,j,k)&m_BoundingBox(i,j,kPlus1)&m_BoundingBox(iPlus1,j,kPlus1)&m_BoundingBox(i,j,kPlus2))&&
                       (
                        (absStructureParity!=1)?
                         (
                          (
                           2*(
                              (char)(chessBoard3DForParityCheck(i,j,k))+
                              (char)(chessBoard3DForParityCheck(i,j,kPlus1))+
                              (char)(chessBoard3DForParityCheck(iPlus1,j,kPlus1))+
                              (char)(chessBoard3DForParityCheck(i,j,kPlus2))
                             )-4
                          )==piecesPAndTParity
                         ) : true
                       )
                      )
                      addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)4),i,j,k,i,j,kPlus1,iPlus1,j,kPlus1,i,j,kPlus2);

                    if(
                       (m_BoundingBox(iPlus1,j,k)&m_BoundingBox(i,j,kPlus1)&m_BoundingBox(iPlus1,j,kPlus1)&m_BoundingBox(iPlus1,j,kPlus2))&&
                       (
                        (absStructureParity!=1)?
                         (
                          (
                           2*(
                              (char)(chessBoard3DForParityCheck(iPlus1,j,k))+
                              (char)(chessBoard3DForParityCheck(i,j,kPlus1))+
                              (char)(chessBoard3DForParityCheck(iPlus1,j,kPlus1))+
                              (char)(chessBoard3DForParityCheck(iPlus1,j,kPlus2))
                             )-4
                          )==piecesPAndTParity
                         ) : true
                       )
                      )
                      addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)4),iPlus1,j,k,i,j,kPlus1,iPlus1,j,kPlus1,iPlus1,j,kPlus2);

// Fin du calcul des positions de T dans la structure  rsoudre.

// Dbut du calcul des positions de Z dans la structure  rsoudre.

                    if(m_BoundingBox(i,j,k)&m_BoundingBox(i,j,kPlus1)&m_BoundingBox(iPlus1,j,kPlus1)&m_BoundingBox(iPlus1,j,kPlus2))
                      addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)6),i,j,k,i,j,kPlus1,iPlus1,j,kPlus1,iPlus1,j,kPlus2);

                    if(m_BoundingBox(iPlus1,j,k)&m_BoundingBox(i,j,kPlus1)&m_BoundingBox(iPlus1,j,kPlus1)&m_BoundingBox(i,j,kPlus2))
                      addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)6),iPlus1,j,k,i,j,kPlus1,iPlus1,j,kPlus1,i,j,kPlus2);

// Fin du calcul des positions de Z dans la structure  rsoudre.

                    }

// Dbut du calcul des positions de V dans la structure  rsoudre.

                  if(
                     (m_BoundingBox(i,j,k)&m_BoundingBox(i,j,kPlus1)&m_BoundingBox(iPlus1,j,kPlus1))&&
                     (
                      (
                       2*(
                          (char)(chessBoard3DForParityCheck(i,j,k))+
                          (char)(chessBoard3DForParityCheck(i,j,kPlus1))+
                          (char)(chessBoard3DForParityCheck(iPlus1,j,kPlus1))
                         )-3
                      )==pieceVParity
                     )
                    )
                    addNewPosToVList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,i,j,k,i,j,kPlus1,iPlus1,j,kPlus1);

                  if(
                     (m_BoundingBox(i,j,k)&m_BoundingBox(iPlus1,j,k)&m_BoundingBox(i,j,kPlus1))&&
                     (
                      (
                       2*(
                          (char)(chessBoard3DForParityCheck(i,j,k))+
                          (char)(chessBoard3DForParityCheck(iPlus1,j,k))+
                          (char)(chessBoard3DForParityCheck(i,j,kPlus1))
                         )-3
                      )==pieceVParity
                     )
                    )
                    addNewPosToVList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,i,j,k,iPlus1,j,k,i,j,kPlus1);

                  if(
                     (m_BoundingBox(i,j,k)&m_BoundingBox(iPlus1,j,k)&m_BoundingBox(iPlus1,j,kPlus1))&&
                     (
                      (
                       2*(
                          (char)(chessBoard3DForParityCheck(i,j,k))+
                          (char)(chessBoard3DForParityCheck(iPlus1,j,k))+
                          (char)(chessBoard3DForParityCheck(iPlus1,j,kPlus1))
                         )-3
                      )==pieceVParity
                     )
                    )
                    addNewPosToVList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,i,j,k,iPlus1,j,k,iPlus1,j,kPlus1);

                  if(
                     (m_BoundingBox(iPlus1,j,k)&m_BoundingBox(i,j,kPlus1)&m_BoundingBox(iPlus1,j,kPlus1))&&
                     (
                      (
                       2*(
                          (char)(chessBoard3DForParityCheck(iPlus1,j,k))+
                          (char)(chessBoard3DForParityCheck(i,j,kPlus1))+
                          (char)(chessBoard3DForParityCheck(iPlus1,j,kPlus1))
                         )-3
                      )==pieceVParity
                     )
                    )
                    addNewPosToVList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,iPlus1,j,k,i,j,kPlus1,iPlus1,j,kPlus1);

// Fin du calcul des positions de V dans la structure  rsoudre.

                  }

                if((jPlus1<N2)&&(kPlus1<N3))
                  {

                  if(jPlus2<N2)
                    {

// Dbut du calcul des positions de L dans la structure  rsoudre.

                    if(m_BoundingBox(i,j,k)&m_BoundingBox(i,j,kPlus1)&m_BoundingBox(i,jPlus1,k)&m_BoundingBox(i,jPlus2,k))
                      addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)2),i,j,k,i,j,kPlus1,i,jPlus1,k,i,jPlus2,k);

                    if(m_BoundingBox(i,j,k)&m_BoundingBox(i,j,kPlus1)&m_BoundingBox(i,jPlus1,kPlus1)&m_BoundingBox(i,jPlus2,kPlus1))
                      addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)2),i,j,k,i,j,kPlus1,i,jPlus1,kPlus1,i,jPlus2,kPlus1);

                    if(m_BoundingBox(i,j,k)&m_BoundingBox(i,jPlus1,k)&m_BoundingBox(i,jPlus2,k)&m_BoundingBox(i,jPlus2,kPlus1))
                      addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)2),i,j,k,i,jPlus1,k,i,jPlus2,k,i,jPlus2,kPlus1);

                    if(m_BoundingBox(i,j,kPlus1)&m_BoundingBox(i,jPlus1,kPlus1)&m_BoundingBox(i,jPlus2,k)&m_BoundingBox(i,jPlus2,kPlus1))
                      addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)2),i,j,kPlus1,i,jPlus1,kPlus1,i,jPlus2,k,i,jPlus2,kPlus1);

// Fin du calcul des positions de L dans la structure  rsoudre.

// Dbut du calcul des positions de T dans la structure  rsoudre.

                    if(
                       (m_BoundingBox(i,j,k)&m_BoundingBox(i,jPlus1,k)&m_BoundingBox(i,jPlus1,kPlus1)&m_BoundingBox(i,jPlus2,k))&&
                       (
                        (absStructureParity!=1)?
                         (
                          (
                           2*(
                              (char)(chessBoard3DForParityCheck(i,j,k))+
                              (char)(chessBoard3DForParityCheck(i,jPlus1,k))+
                              (char)(chessBoard3DForParityCheck(i,jPlus1,kPlus1))+
                              (char)(chessBoard3DForParityCheck(i,jPlus2,k))
                             )-4
                          )==piecesPAndTParity
                         ) : true
                       )
                      )
                      addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)4),i,j,k,i,jPlus1,k,i,jPlus1,kPlus1,i,jPlus2,k);

                    if(
                       (m_BoundingBox(i,j,kPlus1)&m_BoundingBox(i,jPlus1,k)&m_BoundingBox(i,jPlus1,kPlus1)&m_BoundingBox(i,jPlus2,kPlus1))&&
                       (
                        (absStructureParity!=1)?
                         (
                          (
                           2*(
                              (char)(chessBoard3DForParityCheck(i,j,kPlus1))+
                              (char)(chessBoard3DForParityCheck(i,jPlus1,k))+
                              (char)(chessBoard3DForParityCheck(i,jPlus1,kPlus1))+
                              (char)(chessBoard3DForParityCheck(i,jPlus2,kPlus1))
                             )-4
                          )==piecesPAndTParity
                         ) : true
                       )
                      )
                      addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)4),i,j,kPlus1,i,jPlus1,k,i,jPlus1,kPlus1,i,jPlus2,kPlus1);

// Fin du calcul des positions de T dans la structure  rsoudre.

// Dbut du calcul des positions de Z dans la structure  rsoudre.

                    if(m_BoundingBox(i,j,k)&m_BoundingBox(i,jPlus1,k)&m_BoundingBox(i,jPlus1,kPlus1)&m_BoundingBox(i,jPlus2,kPlus1))
                      addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)6),i,j,k,i,jPlus1,k,i,jPlus1,kPlus1,i,jPlus2,kPlus1);

                    if(m_BoundingBox(i,j,kPlus1)&m_BoundingBox(i,jPlus1,k)&m_BoundingBox(i,jPlus1,kPlus1)&m_BoundingBox(i,jPlus2,k))
                      addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)6),i,j,kPlus1,i,jPlus1,k,i,jPlus1,kPlus1,i,jPlus2,k);

// Fin du calcul des positions de Z dans la structure  rsoudre.

                    }

                  if(kPlus2<N3)
                    {

// Dbut du calcul des positions de L dans la structure  rsoudre.

                    if(m_BoundingBox(i,j,k)&m_BoundingBox(i,j,kPlus1)&m_BoundingBox(i,j,kPlus2)&m_BoundingBox(i,jPlus1,kPlus2))
                      addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)2),i,j,k,i,j,kPlus1,i,j,kPlus2,i,jPlus1,kPlus2);

                    if(m_BoundingBox(i,j,k)&m_BoundingBox(i,jPlus1,k)&m_BoundingBox(i,jPlus1,kPlus1)&m_BoundingBox(i,jPlus1,kPlus2))
                      addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)2),i,j,k,i,jPlus1,k,i,jPlus1,kPlus1,i,jPlus1,kPlus2);

                    if(m_BoundingBox(i,j,k)&m_BoundingBox(i,j,kPlus1)&m_BoundingBox(i,j,kPlus2)&m_BoundingBox(i,jPlus1,k))
                      addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)2),i,j,k,i,j,kPlus1,i,j,kPlus2,i,jPlus1,k);

                    if(m_BoundingBox(i,j,kPlus2)&m_BoundingBox(i,jPlus1,k)&m_BoundingBox(i,jPlus1,kPlus1)&m_BoundingBox(i,jPlus1,kPlus2))
                      addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)2),i,j,kPlus2,i,jPlus1,k,i,jPlus1,kPlus1,i,jPlus1,kPlus2);

// Fin du calcul des positions de L dans la structure  rsoudre.

// Dbut du calcul des positions de T dans la structure  rsoudre.

                    if(
                       (m_BoundingBox(i,j,k)&m_BoundingBox(i,j,kPlus1)&m_BoundingBox(i,j,kPlus2)&m_BoundingBox(i,jPlus1,kPlus1))&&
                       (
                        (absStructureParity!=1)?
                         (
                          (
                           2*(
                              (char)(chessBoard3DForParityCheck(i,j,k))+
                              (char)(chessBoard3DForParityCheck(i,j,kPlus1))+
                              (char)(chessBoard3DForParityCheck(i,j,kPlus2))+
                              (char)(chessBoard3DForParityCheck(i,jPlus1,kPlus1))
                             )-4
                          )==piecesPAndTParity
                         ) : true
                       )
                      )
                      addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)4),i,j,k,i,j,kPlus1,i,j,kPlus2,i,jPlus1,kPlus1);

                    if(
                       (m_BoundingBox(i,j,kPlus1)&m_BoundingBox(i,jPlus1,k)&m_BoundingBox(i,jPlus1,kPlus1)&m_BoundingBox(i,jPlus1,kPlus2))&&
                       (
                        (absStructureParity!=1)?
                         (
                          (
                           2*(
                              (char)(chessBoard3DForParityCheck(i,j,kPlus1))+
                              (char)(chessBoard3DForParityCheck(i,jPlus1,k))+
                              (char)(chessBoard3DForParityCheck(i,jPlus1,kPlus1))+
                              (char)(chessBoard3DForParityCheck(i,jPlus1,kPlus2))
                             )-4
                          )==piecesPAndTParity
                         ) : true
                       )
                      )
                      addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)4),i,j,kPlus1,i,jPlus1,k,i,jPlus1,kPlus1,i,jPlus1,kPlus2);

// Fin du calcul des positions de T dans la structure  rsoudre.

// Dbut du calcul des positions de Z dans la structure  rsoudre.

                    if(m_BoundingBox(i,j,k)&m_BoundingBox(i,j,kPlus1)&m_BoundingBox(i,jPlus1,kPlus1)&m_BoundingBox(i,jPlus1,kPlus2))
                      addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)6),i,j,k,i,j,kPlus1,i,jPlus1,kPlus1,i,jPlus1,kPlus2);

                    if(m_BoundingBox(i,j,kPlus1)&m_BoundingBox(i,j,kPlus2)&m_BoundingBox(i,jPlus1,k)&m_BoundingBox(i,jPlus1,kPlus1))
                      addNewPosToPieceList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,((unsigned char)6),i,j,kPlus1,i,j,kPlus2,i,jPlus1,k,i,jPlus1,kPlus1);

// Fin du calcul des positions de Z dans la structure  rsoudre.

                    }

// Dbut du calcul des positions de V dans la structure  rsoudre.

                  if(
                     (m_BoundingBox(i,j,k)&m_BoundingBox(i,j,kPlus1)&m_BoundingBox(i,jPlus1,kPlus1))&&
                     (
                      (
                       2*(
                          (char)(chessBoard3DForParityCheck(i,j,k))+
                          (char)(chessBoard3DForParityCheck(i,j,kPlus1))+
                          (char)(chessBoard3DForParityCheck(i,jPlus1,kPlus1))
                         )-3
                      )==pieceVParity
                     )
                    )
                    addNewPosToVList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,i,j,k,i,j,kPlus1,i,jPlus1,kPlus1);

                  if(
                     (m_BoundingBox(i,j,k)&m_BoundingBox(i,j,kPlus1)&m_BoundingBox(i,jPlus1,k))&&
                     (
                      (
                       2*(
                          (char)(chessBoard3DForParityCheck(i,j,k))+
                          (char)(chessBoard3DForParityCheck(i,j,kPlus1))+
                          (char)(chessBoard3DForParityCheck(i,jPlus1,k))
                         )-3
                      )==pieceVParity
                     )
                    )
                    addNewPosToVList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,i,j,k,i,j,kPlus1,i,jPlus1,k);

                  if(
                     (m_BoundingBox(i,j,k)&m_BoundingBox(i,jPlus1,k)&m_BoundingBox(i,jPlus1,kPlus1))&&
                     (
                      (
                       2*(
                          (char)(chessBoard3DForParityCheck(i,j,k))+
                          (char)(chessBoard3DForParityCheck(i,jPlus1,k))+
                          (char)(chessBoard3DForParityCheck(i,jPlus1,kPlus1))
                         )-3
                      )==pieceVParity
                     )
                    )
                    addNewPosToVList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,i,j,k,i,jPlus1,k,i,jPlus1,kPlus1);

                  if(
                     (m_BoundingBox(i,j,kPlus1)&m_BoundingBox(i,jPlus1,k)&m_BoundingBox(i,jPlus1,kPlus1))&&
                     (
                      (
                       2*(
                          (char)(chessBoard3DForParityCheck(i,j,kPlus1))+
                          (char)(chessBoard3DForParityCheck(i,jPlus1,k))+
                          (char)(chessBoard3DForParityCheck(i,jPlus1,kPlus1))
                         )-3
                      )==pieceVParity
                     )
                    )
                    addNewPosToVList(N2,N3,N1xN2xN3,m_PiecesPositionsLists,i,j,kPlus1,i,jPlus1,k,i,jPlus1,kPlus1);

// Fin du calcul des positions de V dans la structure  rsoudre.

                  }

                }

              }

            }

          }

//  Si la matrice m_PiecesPositionsLists est vide, alors il n'y a aucun moyen de placer quelque pice que ce soit dans la structure
//   rsoudre. Donc m_HasNoSolution est assign  true.
        m_HasNoSolution=m_HasNoSolution&&(m_PiecesPositionsLists.empty());

        }



      }



    }



  }
