#ifndef SOMA_LOADER_HPP
#define SOMA_LOADER_HPP



#include "../../utils/general/BitArray3D.hpp"



namespace soma
  {



  namespace utils
    {

    namespace general
      {

//  Dclaration anticipe de la classe CBitArray3D. C'est inutile ici car un membre de la classe
//  CSomaLoader est de type CBitArray3D (composition). C'est plus pour indiquer qu'on va utiliser
//  la classe CBitArray3D.
      class CBitArray3D;

//  Dclaration de la fonction isOfType.
      template
        <
        typename T
        >
      inline const bool isOfType
                               (
                               T& toAssign,
                               const std::string& str
                               );

      }

    }



  namespace solver
    {



    namespace soma_structure
      {



/*
  La classe CSomaLoader permet de charger une figure respectant le format de fichier dcrit sur la page du dfi (http://c.developpez.com/defis/5-Cube-Soma/).
  Elle possde un seul constructeur dont l'unique argument est une chane de charactres constante, de type const char*, contenant le nom du fichier  charger.
  Ce fichier doit respecter scrupuleusement le format impos par le rglement du jeu. Il est uniquement permis de sauter des lignes entre chaque ligne de coordonnes
  (mais pas aprs la dernire ligne) ou de mettre des espaces aprs chaque coordonne au sein d'une ligne (mais pas aprs la dernire coordonne).
  Il n'est pas obligatoire qu'il y ait exactement 27 cubes de coordonnes diffrentes, mais il doit y avoir exactement 27 cubes de coordonnes au format valide
  (x, y, et z spars par des virgules et compris entre 0 et 9 inclus).
*/
      class CSomaLoader
        {

        private :

/*
    Donne membre prive :
      - type : soma::utils::general::CBitArray3.
      - nom : m_BoundingBox.
      - m_BoundingBox est la bote englobante contenant la structure une fois charge. Elle est centre en (0,0,0). Elle est code par un tableau de boolens
         3 dimensions. Si un cube appartenant  la structure est prsent  la position (i,j,k), alors m_BoundingBox(i,j,k)==true. Sinon, m_BoundingBox(i,j,k)==false.
*/
          soma::utils::general::CBitArray3D m_BoundingBox;



//  Copie d'un objet de type CSomaLoader interdite.
          CSomaLoader
                    (
                    const CSomaLoader&
                    );



//  Assignation d'un objet de type CSomaLoader interdite.
          void operator=
                       (
                       const CSomaLoader&
                       );


        public :

/*
     Unique constructeur public de la classe CSomaLoader. Reoit le nom du fichier  charger.

     Argument :
       - type : const char*.
       - nom : fileName.
       - fileName contient le nom du fichier  charger.
*/
          CSomaLoader
                    (
                    const char* fileName
                    );



/*
    getBoundingBox est une fonction membre publique const.

    Argument : Aucun.

    Retour :
      - type : rfrence constante sur soma::utils::general::CBitArray3D.
      - nom : m_BoundingBox.
      - Cette fonction membre publique est l'accesseur au membre m_BoundingBox.
*/
          inline const soma::utils::general::CBitArray3D& getBoundingBox() const
            {
            return
                 (
                 m_BoundingBox
                 );
            }

        };



      }



    }



  }



#endif  //  SOMA_LOADER_HPP
