


namespace soma
  {



  namespace utils
    {



    namespace general
      {



      CBitArray3D::CBitArray3D
                             (
                             CBitArray3D::const_size_type N1,
                             CBitArray3D::const_size_type N2,
                             CBitArray3D::const_size_type N3
                             )
                             : m_Size1(N1),
                               m_Size2(N2),
                               m_Size3(N3),
                               m_Data(N1*N2*N3)
        {}



      CBitArray3D::CBitArray3D
                             (
                             CBitArray3D::const_size_type N1,
                             CBitArray3D::const_size_type N2,
                             CBitArray3D::const_size_type N3,
                             CBitArray3D::const_value_type fillVal
                             )
                             : m_Size1(N1),
                               m_Size2(N2),
                               m_Size3(N3),
                               m_Data(N1*N2*N3,fillVal)
        {}



      inline CBitArray3D& CBitArray3D::resize
                                            (
                                            CBitArray3D::const_size_type newSize1,
                                            CBitArray3D::const_size_type newSize2,
                                            CBitArray3D::const_size_type newSize3,
                                            const bool fillVal
                                            )
        {
        m_Data.resize
                    (
                    (m_Size1=newSize1)*(m_Size2=newSize2)*(m_Size3=newSize3),
                    fillVal
                    );
        return
             (
             *this
             );
        }



      inline const CBitArray3D CBitArray3D::generate3DChessBoard
                                                               (
                                                               const_size_type N1,
                                                               const_size_type N2,
                                                               const_size_type N3,
                                                               const_value_type firstVal
                                                               )
        {
        CBitArray3D arr3D
                        (
                        N1,
                        N2,
                        N3
                        );
        CBitArray3D::size_type j=1,
                               k=1;
        arr3D(0,0,0)=firstVal;
        for(;k<N3;++k)
          (arr3D(0,0,k)=arr3D(0,0,k-1)).flip();
        for(;j<N2;++j)
          for(k=0;k<N3;++k)
            (arr3D(0,j,k)=arr3D(0,j-1,k)).flip();
        for(CBitArray3D::size_type i=1;i<N1;++i)
          for(j=0;j<N2;++j)
            for(k=0;k<N3;++k)
              (arr3D(i,j,k)=arr3D(i-1,j,k)).flip();
        return
             (
             arr3D
             );
        }



      inline CBitArray3D::reference CBitArray3D::operator()
                                                          (
                                                          const_size_type i,
                                                          const_size_type j,
                                                          const_size_type k
                                                          )
        {
        return
             (
             m_Data[m_Size3*(m_Size2*i+j)+k]
             );
        }



      inline CBitArray3D::const_reference CBitArray3D::operator()
                                                                (
                                                                const_size_type i,
                                                                const_size_type j,
                                                                const_size_type k
                                                                ) const
        {
        return
             (
             m_Data[m_Size3*(m_Size2*i+j)+k]
             );
        }



      inline CBitArray3D::reference CBitArray3D::at
                                                  (
                                                  const_size_type i,
                                                  const_size_type j,
                                                  const_size_type k
                                                  )
        {
        if((i>m_Size1)||(j>m_Size2)||(k>m_Size3))
          throw
              (
              std::out_of_range
                              (
                              "Indices not in ranges!"
                              )
              );
        return
             (
             (*this)(i,j,k)
             );
        }



      inline CBitArray3D::const_reference CBitArray3D::at
                                                        (
                                                        const_size_type i,
                                                        const_size_type j,
                                                        const_size_type k
                                                        ) const
        {
        if((i>m_Size1)||(j>m_Size2)||(k>m_Size3))
          throw
              (
              std::out_of_range
                              (
                              "Indices not in ranges!"
                              )
              );
        return
             (
             (*this)(i,j,k)
             );
        }



      inline CBitArray3D& CBitArray3D::flip()
        {
        m_Data.flip();
        return
             (
             *this
             );
        }



      inline const CBitArray3D CBitArray3D::operator~() const
        {
        CBitArray3D arr3D
                        (
                        m_Size1,
                        m_Size2,
                        m_Size3
                        );
        const_size_type _size=size();
        for(size_type i=0;i<_size;++i)
          (arr3D.m_Data[i]=m_Data[i]).flip();
        return
             (
             arr3D
             );
        }



      inline CBitArray3D& CBitArray3D::swap
                                          (
                                          CBitArray3D& arr3D
                                          )
        {
        if(this!=&arr3D)
          {
          m_Data.swap
                    (
                    arr3D.m_Data
                    );
          /*CBitArray3D::const_size_type _size1=std::min
                                                   (
                                                   size1,
                                                   arr3D.size1
                                                   ),
                                       _size2=std::min
                                                   (
                                                   size2,
                                                   arr3D.size2
                                                   ),
                                       _size3=std::min
                                                   (
                                                   size3,
                                                   arr3D.size3
                                                   );
          for(size_type i=0;i<dim1;++i)
            for(size_type j=0;j<dim2;++j)
              for(size_type k=0;k<dim3;++k)
                CBitArray3D::swap
                                (
                                (*this)(i,j,k),
                                arr3D(i,j,k)
                                );*/
          }
        return
             (
             *this
             );
        }



      inline void CBitArray3D::swap
                                  (
                                  CBitArray3D::reference x,
                                  CBitArray3D::reference y
                                  )
        {
        std::vector<bool>::swap
                              (
                              x,
                              y
                              );
        }



      }



    }



  }
